#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------

#-------------------------------COMENTARIO GLOBAL---------------
#---------------------------------------------------------------
# PROPOSITO.
#-----------
# Funciones centralizadas para el sistema de tags
#---------------------------------------------------------------

#---------------------------------------------------------------
# DECLARACIONES GLOBALES.
#------------------------

package lib_tags;
use strict;
use utf8;
use prontus_varglb;
use DBI;
use JSON;

use lib_prontus;
use glib_dbi_03;
use glib_fildir_03;

our $MAX_ARTICS_PER_SEARCH  = 20;
our $MAX_TAGS_SEARCH_RESULT = 20;

# ---------------------------------------------------------------
# SUB-RUTINAS.
# ---------------------------------------------------------------

# Verifica que el nombre no exista en la tabla.
sub check_nom {
    my ($db, $id, $nombre) = ($_[0], $_[1], $_[2]);
    my @params;
    my $sql;

    if ($id) {
        $sql = "select count(TAGS_TAG) from TAGS where lower(TAGS_TAG) = lower(?) and TAGS_ID <> ?";
        push @params, $nombre;
        push @params, $id;
    } else {
        $sql = "select count(TAGS_TAG) from TAGS where lower(TAGS_TAG) = lower(?)";
        push @params, $nombre;
    }

    my $cant;
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($db, $sql, \@params, \$cant);

    $salida->fetch;
    $salida->finish;

    if ($cant > 0) {
        return 'Nombre de ítem ya existe.';
    }

    return '';
}

# ---------------------------------------------------------------
sub get_tags_for_fid {
    my $base = $_[0];
    my $ts   = $_[1];
    my $tags_order = $_[2];
    my @params;

    # tags asignados
    my $sql = "select TAGSART_IDTAGS,TAGS_TAG from TAGSART join TAGS on TAGS_ID=TAGSART_IDTAGS where TAGSART_IDART = ?";
    push @params, $ts;

    if ($tags_order) {
        $sql .= " order by FIELD(TAGS_ID, ?);";
        push @params, $tags_order;
    } else {
        $sql .= " order by TAGS_TAG asc;";
    }

    my $tags_query = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    my ($idtag_asignado, $tag_name);

    $tags_query->bind_columns(undef, \($idtag_asignado, $tag_name));

    # Se leen los tags de la BD
    my $tags_asignados = '';

    while ($tags_query->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag_name);
        $tag_name = &lib_prontus::escape_xml($tag_name);
        $tags_asignados = $tags_asignados . "$idtag_asignado|$tag_name,";
    }

    $tags_asignados =~ s/,$//;
    $tags_query->finish;

    # warn("tags_asignados[$tags_asignados]");
    return $tags_asignados;
}

# ---------------------------------------------------------------
sub get_last_tags {
    my ($base, $last_tags_max) = ($_[0], $_[1]);
    my @params;
    # los ultimos tags ingresados
    my $sql = "select TAGS_ID,TAGS_TAG from TAGS where TAGS_MOSTRAR = 1 order by TAGS_ID desc limit ?";

    push @params, $last_tags_max;

    my $tab_output = &glib_dbi_03::ejecutar_sql_param($base, $sql, \@params);
    my ($idtag, $tag_name);

    $tab_output->bind_columns(undef, \($idtag, $tag_name));
    my %hash_tags;

    while ($tab_output->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag_name);
        $tag_name = &lib_prontus::escape_xml($tag_name);
        $hash_tags{$idtag} = $tag_name;
    }

    my $last_tags = '';

    foreach my $tagid (sort { &lib_tags::normalize4sort($hash_tags{$a}) cmp &lib_tags::normalize4sort($hash_tags{$b}) } keys %hash_tags) {
        my $shortname = $hash_tags{$tagid};
        if (length($shortname) > 18) {
            $shortname = substr($shortname, 0, 18) . '...';
        }
        $last_tags = $last_tags . '<div class="unused"><a href="#tag' . $tagid . '" rel="item' . $tagid . '" title="Agregar Tag: ' . $hash_tags{$tagid} . '" alt="' . $hash_tags{$tagid} . '">' . $shortname . '</a></div> ';
    }

    return $last_tags;
}

# ---------------------------------------------------------------
sub get_all_tags {
    my $base = $_[0];
    my %hash_tags;

    my $sql = "select TAGS_ID, TAGS_TAG from TAGS where TAGS_MOSTRAR = 1 order by TAGS_TAG asc";
    my $tab_output = &glib_dbi_03::ejecutar_sql($base, $sql);

    my ($idtag, $tag_name);
    $tab_output->bind_columns(undef, \($idtag, $tag_name));

    while ($tab_output->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag_name);
        $hash_tags{$idtag} = $tag_name;
    }
    return %hash_tags;
}

# ---------------------------------------------------------------
sub normalize4sort {
    my $in = $_[0];
    $in = lc($in);
    $in =~ tr/Ñ/ñ/;    # lc probably didn't catch this
    $in =~ tr/áéíóúü/aeiouu/;
    $in =~ tr< abcdefghijklmnñopqrstuvwxyz >
        < \x01-\x1A >;    # 1B = 27
    return $in;
}

# ---------------------------------------------------------------
sub get_tag_noms {
    my $base   = shift;
    my $tag_id = shift;
    my %tag_noms;
    my @params;
    my $sql = "select TAGS_TAG, TAGS_NOM4VISTAS from TAGS where TAGS_ID = ?";

    push @params, $tag_id;

    my $tag;
    my $nom4vistas;
    my $salida = &glib_dbi_03::ejecutar_sql_bind_param($base, $sql, \@params, \($tag, $nom4vistas));

    while ($salida->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag);
        &glib_str_03::correct_encode_utf8_string($nom4vistas);

        $tag_noms{''} = &lib_prontus::escape_html($tag);    # nombre en la vista default o ppal

        while ($nom4vistas =~ /(^|\n)(\w+)\t(.*?)($|\n)/sg) {
            my $vista = $2;
            my $nom   = $3;

            $tag_noms{$vista} = &lib_prontus::escape_html($nom);    # nombre en cada vista
        }
    }

    $salida->finish;

    return %tag_noms;
}

# Dada una id de tag, devuelve la URL custom, si existe en db.
# De otro modo devuelve la URL estándar de tagport
# Intenta leer el archivo json de tags.
# @param int $tag_id
# @return string
sub get_tag_url {
    my $tag_id = shift;
    my ($sql, $salida, $tag_url, $url);

    my @params = ($tag_id);
    $sql = "SELECT TAGS_PORT FROM TAGS WHERE TAGS_ID = ?";
    $salida = &glib_dbi_03::ejecutar_sql_bind_param($prontus_varglb::BD_CONN{$prontus_varglb::PRONTUS_ID}, $sql, \@params, \($tag_url));

    while ($salida->fetch) {
        if ($tag_url) {
            $url = $tag_url;
        } else {
            $url = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/tag/port/all/tagport_".$tag_id."_1.html";
        }
    }
    $salida->finish;
    return $url;
}

#---------------------------------------------------------------
sub generar_json {
    my $bd = shift;
    my ($sql, $salida, $tag_id, $tag_nom);

    my %tags_url = ();
    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'FRIENDLY_URLS_VERSION'} eq '5') {
        my ($salida2, $urlf5_subtypeid, $urlf5_file, $urlf5_url);
        my $sql2 = "SELECT URLF5_SUBTYPEID, URLF5_FILE, URLF5_URL FROM URLF5 WHERE URLF5_TYPE = ?";
        my @params = ('tag');
        $salida2 = &glib_dbi_03::ejecutar_sql_bind_param($bd, $sql2, \@params, \($urlf5_subtypeid, $urlf5_file, $urlf5_url));

        while ($salida2->fetch) {
            $urlf5_file =~ s/\/$prontus_varglb::PRONTUS_ID//;
            $tags_url{$urlf5_subtypeid}{$urlf5_file} = $urlf5_url;
        }
        $salida2->finish;
    }

    $sql = "SELECT TAGS_ID, TAGS_TAG FROM TAGS WHERE TAGS_MOSTRAR = 1";
    $salida = &glib_dbi_03::ejecutar_sql_bind($bd, $sql, \($tag_id, $tag_nom));
    my @tags;
    while ($salida->fetch) {
        &glib_str_03::correct_encode_utf8_string($tag_nom);
        my %urls = ();
        if (defined($tags_url{$tag_id})) {
            %urls = %{$tags_url{$tag_id}};
        }
        push @tags, {'id' => $tag_id, 'nom' => $tag_nom, 'urls' => \%urls};
    }

    $salida->finish;

    my $str_json = encode_json(\@tags);
    utf8::decode($str_json);

    my $dst_path = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}$prontus_varglb::DIR_DATA4FIDS";

    &glib_fildir_03::check_dir($dst_path);
    &glib_fildir_03::write_file("$dst_path/tags.json", $str_json);
}

1;
